#ifndef CHESSY_CHESSGAME_H
#define CHESSY_CHESSGAME_H

#include <vector>
#include <string>

#include "Board.h"

namespace chessy
{
    class ChessGame;
}


class chessy::ChessGame
{
    Board board;

    bool canCastleQueenSide[2] = { true, true };
    bool canCastleKingSide [2] = { true, true };

    short reversibleHalfMoves =     0;

    //! -1 for no en passant possible
    Index enPassant =              -1;

    Side turn =                     WHITE_SIDE;

    int moveCount =                 1;

    /*const static char WHITE =       0;
    const static char BLACK =       1;*/


public:

    ChessGame(void);
    ChessGame(const std::string& fenString);
    ChessGame(const ChessGame&) = default;
    ~ChessGame(void) = default;

    bool isValidMove(const Move& move) const;
    std::vector<Move> getValidMoves(void) const;

    inline const Board& getBoard(void) const { return board; }
    inline       Board& getBoard(void)       { return board; }

    inline bool getCanCastleKingSide(Side side) const
    {
        return canCastleKingSide[side];
    }

    inline bool getCanCastleQueenSide(Side side) const
    {
        return canCastleQueenSide[side];
    }

    void move(Move move);
    inline Side getTurn(void) const { return turn; }

    void loadFromFen(const std::string& fenString);
    std::string generateFen(void) const;
};

#endif // CHESSY_CHESSGAME_H

